import React, { useState, useEffect, useRef } from 'react';
import { SimpleAccordion } from "../common/Accordion/SimpleAccordion";
import { Button, makeStyles, TextField, Tooltip, Typography } from "@material-ui/core";
import { Autocomplete, Alert } from "@material-ui/lab";
import { useAllowedGitProjects, useNamingNames, useUpdateIoc, useUserNames } from "../../api/SwaggerApi";
import { useTypingTimer } from "../common/SearchBoxFilter/TypingTimer";
import { ConfirmationDialog } from "../dialog/ConfirmationDialog";
import { SimpleModal } from "../../components/common/SimpleModal/SimpleModal";
import AccessControl from "../../components/auth/AccessControl";

const useStyles = makeStyles((theme) => ({
    textField: {
        marginBottom: theme.spacing(1),
    },
}));

export default function IOCDetailAdmin({ ioc, getIOC, resetTab, buttonDisabled }) {
    const classes = useStyles();

    const [gitId, setGitId] = useState(ioc.gitProjectId);
    const [allowedGitProjects, getAllowedGitProjects] = useAllowedGitProjects([]);
    const [nameQuery, onNameKeyUp] = useTypingTimer({ interval: 500 });
    const [names, getNames] = useNamingNames();
    const [name, setName] = useState({ id: ioc.externalNameId, description: ioc.description, name: ioc.namingName });
    const [owner, setOwner] = useState(ioc.owner);
    const [ownerQuery] = useState();
    const [users, getUsers] = useUserNames();

    //for the dialog
    const [adHocDialogOpen, setAdHocDialogOpen] = useState(false);
    const [adHocDialogTitle, setAdHocDiatlogTitle] = useState();
    const [adHocDialogDescription, setAdHocDialogDescription] = useState();
    const [error, setError] = useState();
    const callbackRef = useRef();

    function onError(message) {
        setError(message);
    }  

    const [uioc, actionUpdateIoc] = useUpdateIoc(ioc.id, onError);

    useEffect(() => {
        getAllowedGitProjects();
    }, [getAllowedGitProjects]);

    useEffect(() => getNames(nameQuery), [nameQuery, getNames]);

    useEffect(() => getUsers(ownerQuery), [ownerQuery, getUsers]);

    //when user clicks Submit button a dialog should open
    const onSubmit = (event) => {
        event.preventDefault();

        openModifyModal();
    }

    //Setting up dialog properties
    const openModifyModal = () => {
        setAdHocDiatlogTitle("Modifying IOC");
        setAdHocDialogDescription(
            <>
                <Typography style={{ display: 'inline-block' }}>Are you sure want to modify &nbsp;</Typography>
                <Typography style={{ fontFamily: "monospace", display: 'inline-block' }}> {ioc.namingName}? </Typography>
            </>);
        callbackRef.current = modifyIoc;
        setAdHocDialogOpen(true);
    }

    //what to do when modifying IOC
    const modifyIoc = () => {

         actionUpdateIoc({
            owner: owner,
            gitProjectId: gitId,
            externalNameId: name ? parseInt(name.id) : null
        });
    }

    useEffect(() => 
    {
      if(uioc) {
        getIOC();
        resetTab();
      }
    }, [uioc, getIOC, resetTab]);

    const iocIsDeployed = ioc.activeDeployment;

    let nameDisabledTitle = "";
    if (iocIsDeployed) {
        nameDisabledTitle = "IOC cannot be renamed while deployed - please undeploy first"
    }

    function nameAutocomplete(disabled) {
        return (
            <Tooltip title={nameDisabledTitle}>
                <span>
                    <Autocomplete
                        className={classes.textField}
                        autoHighlight
                        id="namingName"
                        options={names}
                        defaultValue={name}
                        getOptionLabel={(option) => { console.log(option?.name); return option?.name }}
                        renderInput={(params) => <TextField {...params} label="IOC name" variant="outlined" required />}
                        onChange={(event, value, reason) => { setName(value); setError(null); }}
                        onInputChange={(event, value, reason) => { event && onNameKeyUp(event.nativeEvent) }}
                        disabled={disabled || iocIsDeployed}
                        autoSelect
                    />
                </span>
            </Tooltip>
        );
    }

    function gitRepoAutocomplete(disabled) {
        return(
            <Autocomplete
                className={classes.textField}
                autoHighlight
                id="gitId"
                options={allowedGitProjects}
                defaultValue={ {
                    "id": gitId,
                    "url": ioc.sourceUrl
                } }
                getOptionLabel={(option) => { return option.url }}
                onChange={(event, value, reason) => {setGitId(value.id); setError(null);}}
                renderInput={(params) => <TextField {...params} label="Git repository" variant="outlined" fullWidth required />} 
                disabled={disabled || iocIsDeployed}
                autoSelect
            />
        );
    }

    function ownerAutocomplete(disabled) {
        return(
            <Autocomplete
                className={classes.textField}
                autoHighlight
                id="owner"
                options={users}
                getOptionLabel={(option) => { return option }}
                defaultValue={owner}
                renderInput={(params) => <TextField {...params} label="Owner" variant="outlined" fullWidth required />}
                onChange={(event, value, reason) => {setOwner(value); setError(null);}}
                disabled={disabled || iocIsDeployed}
                autoSelect
            />
        );
    }

    let disabledButtonTitle = "";
    if (buttonDisabled || ioc.operationInProgress) {
        disabledButtonTitle = "There is an ongoing operation, you can not modify the IOC right now";
    }

    return (
        <>
            <SimpleModal open={adHocDialogOpen} setOpen={setAdHocDialogOpen}>
                <ConfirmationDialog open={adHocDialogOpen} setOpen={setAdHocDialogOpen} title={adHocDialogTitle} description={adHocDialogDescription} callback={callbackRef.current} />
            </SimpleModal>

            <SimpleAccordion summary="Modify IOC" defaultExpanded>
                <form onSubmit={onSubmit} style={{ width: "100%" }}>
                    <AccessControl 
                        allowedRoles={["DeploymentToolAdmin"]}
                        renderNoAccess={() => nameAutocomplete(true)} >
                        {nameAutocomplete(iocIsDeployed)}
                    </AccessControl>

                    <AccessControl 
                        allowedRoles={["DeploymentToolAdmin"]}
                        allowedUsersWithRoles={[{ user: ioc.owner, role: "DeploymentToolIntegrator" }]}
                        renderNoAccess={() => gitRepoAutocomplete(true)} >
                        {gitRepoAutocomplete(false)}
                    </AccessControl>

                    <AccessControl 
                        allowedRoles={["DeploymentToolAdmin", "DeploymentToolManager"]}
                        renderNoAccess={() => ownerAutocomplete(true)} >
                        {ownerAutocomplete(false)}
                    </AccessControl>

                    {error ?
                        <Alert severity="error">{error}</Alert>
                    : <></>}

                    <br /><br />
                    <Tooltip title={disabledButtonTitle}>
                        <span>
                            <Button color="primary" variant="contained" disabled={buttonDisabled || ioc.operationInProgress || iocIsDeployed} type="submit">Modify IOC</Button>
                        </span>
                    </Tooltip>
                </form>
            </SimpleAccordion>
        </>
    )
}